'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

beforeEach(function () {
  jest.resetModules();
});

it('should use http://localhost:8545 by default', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
  var waitForWeb3, web3;
  return _regenerator2.default.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          waitForWeb3 = require('../waitForWeb3').default;
          _context.next = 3;
          return waitForWeb3();

        case 3:
          web3 = _context.sent;


          expect(web3.currentProvider.host).toBe('http://localhost:8545');

        case 5:
        case 'end':
          return _context.stop();
      }
    }
  }, _callee, undefined);
})));

it('should handle a fallbackProvider', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
  var waitForWeb3, web3;
  return _regenerator2.default.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          waitForWeb3 = require('../waitForWeb3').default;
          _context2.next = 3;
          return waitForWeb3({ fallbackProvider: 'Foo' });

        case 3:
          web3 = _context2.sent;


          expect(web3.currentProvider.host).toBe('Foo');

        case 5:
        case 'end':
          return _context2.stop();
      }
    }
  }, _callee2, undefined);
})));

describe('when window.web3 is present', function () {
  beforeAll(function () {
    window.web3 = {
      currentProvider: {
        host: 'Bar'
      }
    };
  });

  it('should take provider from injected web3', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
    var waitForWeb3, web3;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            waitForWeb3 = require('../waitForWeb3').default;
            _context3.next = 3;
            return waitForWeb3();

          case 3:
            web3 = _context3.sent;


            expect(web3.currentProvider.host).toBe('Bar');

          case 5:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  })));

  afterAll(function () {
    delete window.web3;
  });
});

describe('when window takes time to load', function () {
  beforeAll(function () {
    Object.defineProperty(document, 'readyState', {
      value: 'loading',
      writable: true
    });
    window.addEventListener = function (_, resolve) {
      return setTimeout(resolve, 200);
    };
  });

  it('should wait until window loads and fetch web3', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
    var waitForWeb3, web3;
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            waitForWeb3 = require('../waitForWeb3').default;
            _context4.next = 3;
            return waitForWeb3();

          case 3:
            web3 = _context4.sent;


            expect(web3).not.toBeNull();

          case 5:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  })));

  afterAll(function () {
    Object.defineProperty(document, 'readyState', {
      value: 'complete',
      writable: true
    });
    // TODO restore window.addEventListener?
  });
});

describe('when calling multiple times waitForWeb3', function () {
  var waitForWeb3 = require('../waitForWeb3').default;
  var localWeb3 = void 0;

  beforeAll(function () {
    return waitForWeb3().then(function (web3) {
      localWeb3 = web3;
    });
  });

  it('should return the same web3 object upon future call', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5() {
    var web3;
    return _regenerator2.default.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return waitForWeb3();

          case 2:
            web3 = _context5.sent;


            expect(web3).toBe(localWeb3);

          case 4:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, undefined);
  })));
});