'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _web = require('web3');

var _web2 = _interopRequireDefault(_web);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var web3js = void 0;

/**
 * Resolve with web3 instance
 * @param {function} resolve Resolve function to call with web3 as argument
 */
var resolveWeb3 = function resolveWeb3(resolve, opts) {
  var options = (0, _extends3.default)({
    fallbackProvider: 'http://localhost:8545'
  }, opts);
  // Checking if Web3 has been injected by the browser (Mist/MetaMask)
  if (typeof window.web3 !== 'undefined') {
    // Use Mist/MetaMask's provider
    web3js = new _web2.default(window.web3.currentProvider);
  } else {
    // fallback - use your fallback strategy (local node / hosted node + in-dapp id mgmt / fail)
    web3js = new _web2.default(new _web2.default.providers.HttpProvider(options.fallbackProvider));
  }
  resolve(web3js);
};

/**
 * Return a promise which resolves with the web3 instance as soon as it's
 * available
 */
var waitForWeb3 = function waitForWeb3(opts) {
  return new _promise2.default(function (resolve) {
    // If our web3js already exists, resolve immediately
    if (web3js) return resolve(web3js);

    // If an instance of web3 exist in window, resolve immediately
    if (typeof window.web3 !== 'undefined') return resolveWeb3(resolve, opts);

    // If window is already full loaded, resolve immediately
    if (window.document.readyState === 'complete') {
      return resolveWeb3(resolve, opts);
    }

    // Wait until window has fully loaded to resolve web3
    // See https://github.com/MetaMask/faq/blob/master/DEVELOPERS.md#partly_sunny-web3---ethereum-browser-environment-check
    window.addEventListener('load', function () {
      return resolveWeb3(resolve, opts);
    });
  });
};

exports.default = waitForWeb3;