'use strict';

beforeEach(function () {
  jest.resetModules();
});

it('should use http://localhost:8545 by default', function () {
  var waitForWeb3 = require('./waitForWeb3').default;
  expect.assertions(1);
  return waitForWeb3().then(function (web3) {
    expect(web3.currentProvider.host).toBe('http://localhost:8545');
  });
});

it('should handle a fallbackProvider', function () {
  var waitForWeb3 = require('./waitForWeb3').default;
  expect.assertions(1);
  return waitForWeb3({ fallbackProvider: 'Foo' }).then(function (web3) {
    expect(web3.currentProvider.host).toBe('Foo');
  });
});

describe('when window.web3 is present', function () {
  beforeAll(function () {
    window.web3 = {
      currentProvider: {
        host: 'Bar'
      }
    };
  });

  it('should take provider from injected web3', function () {
    var waitForWeb3 = require('./waitForWeb3').default;

    expect.assertions(1);
    return waitForWeb3().then(function (web3) {
      expect(web3.currentProvider.host).toBe('Bar');
    });
  });

  afterAll(function () {
    delete window.web3;
  });
});

describe('when window takes time to load', function () {
  beforeAll(function () {
    Object.defineProperty(document, 'readyState', {
      value: 'loading',
      writable: true
    });
    window.addEventListener = function (_, resolve) {
      return setTimeout(resolve, 200);
    };
  });

  it('should wait until window loads and fetch web3', function () {
    var waitForWeb3 = require('./waitForWeb3').default;

    expect.assertions(1);
    return waitForWeb3().then(function (web3) {
      expect(web3).not.toBeNull();
    });
  });

  afterAll(function () {
    Object.defineProperty(document, 'readyState', {
      value: 'complete',
      writable: true
    });
    // TODO restore window.addEventListener?
  });
});

describe('when calling multiple times waitForWeb3', function () {
  var waitForWeb3 = require('./waitForWeb3').default;
  var localWeb3 = void 0;

  beforeAll(function () {
    return waitForWeb3().then(function (web3) {
      localWeb3 = web3;
    });
  });

  it('should return the same web3 object upon future call', function () {
    expect.assertions(1);
    return waitForWeb3().then(function (web3) {
      expect(web3).toBe(localWeb3);
    });
  });
});